@@ -25,22 +25,41 @@ class EmailService(domain.EmailService):
 
     def add_contact_to_audience(self, contact: domain.Contact, audience: domain.Audience, meta: dict = None,
                                 tags: list = None):
-        if audience.get_member_by_contact(contact) is not None:
-            return
-        audience.add_member(contact, meta, tags)
+        audience_members = self._registry(domain.AudienceMember)
+        audience_member = self._get_audience_member(audience, contact)
+        if audience_member is not None:
+            audience_member.meta.update(meta or {})
+            audience_member.tags.extend(tags or [])
+        else:
+            audience_members.append(domain.AudienceMember(
+                audience=audience.id,
+                contact=contact.id,
+                meta=meta or {},
+                tags=tags or []
+            ))
+
         for service in audience.services:
             self._email_service_factory(service).add_contact_to_audience(contact, audience, meta, tags)
 
     def add_tag_to_audience_member(self, tag: str, audience: domain.Audience, contact: domain.Contact):
-        if tag in audience.get_member_by_contact(contact).tags:
+        audience_member = self._get_audience_member(audience, contact)
+        if audience_member is None or tag in audience_member.tags:
             return
-        audience.get_member_by_contact(contact).tags.append(tag)
+
+        audience_member.tags.append(tag)
         for service in audience.services:
             self._email_service_factory(service).add_tag_to_audience_member(tag, audience, contact)
 
     def remove_tag_from_audience_member(self, tag: str, audience: domain.Audience, contact: domain.Contact):
-        if tag not in audience.get_member_by_contact(contact).tags:
+        audience_member = self._get_audience_member(audience, contact)
+        if audience_member is None or tag not in audience_member.tags:
             return
-        audience.get_member_by_contact(contact).tags.remove(tag)
+
+        audience_member.tags.remove(tag)
         for service in audience.services:
             self._email_service_factory(service).remove_tag_from_audience_member(tag, audience, contact)
+
+    def _get_audience_member(self, audience: domain.Audience, contact: domain.Contact):
+        return self._registry(domain.AudienceMember).find(
+            lambda am: (am.audience == audience.id) & (am.contact == contact.id)
+        )
